;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#xe34f2996cec36242f4cd9c5bd98310bc . "./code.cg2js/cg-cursor-pixmap.fasl")
(#x1ed9ed15e0861142b0bef00dc5053af0 . "./code.cg2js/cg-printing.fasl")
(#x5fb30e0235da29eaff5814f6d2d214e6 . "./code.cg2js/cg-caret.fasl")
(#x9b503e56c350f087d61ed282e35e6a2a . "./code.cg2js/cg-item-list.fasl")
(#x8b6f32e7e7c2443551ca753612a0d4cd . "./code.cg2js/cg-progress-indicator.fasl")
(#xfa9075ffaad4a49d0c30937eed99c350 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#x69647a507ef35d89f030ae08f257742c . "./code.cg2js/cg-outline.fasl")
(#x0ab5577030b173e7fd9d23ccf89671ba . "./code.cg2js/cg-palette.fasl")
(#x7a687ba48a4b62a5d4d1b01ebf2e46a0 . "./code.cg2js/cg-selected-object.fasl")
(#x79d01a3b91c0bfbce719a59e438b1258 . "./code.cg2js/cg-status-bar.fasl")
(#xf25bf4cdaedfcafb8bb68ef14b6631e8 . "./code.cg2js/cg-wrap-string.fasl")
(#xea5e475a88104e948b3053bcaebad92a . "./code.cg2js/cg-fill-texture.fasl")
(#xb8e2c676b0e4304b524dee91bbc9a452 . "./code.cg2js/cg-layout.fasl")
(#x5567896bf6eb90fe8b96a556c1a728c6 . "./code.cg2js/cg-timer.fasl")
(#x85369d849657b421cb6bdef79e43d3f1 . "./code.cg2js/cg-dialog-item.fasl")
(#x3a16e64f7c8d6989b11b47ddbc789dc8 . "./code.cg2js/cg-button.fasl")
(#xb07afeb11f71b536970636d5ddbcdeaf . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#x7ca3b0083efaf8396d7a604e85e61642 . "./code.cg2js/cg-tray.fasl")
(#x6eb1e8119957fec5f8ea7a092a13ba4d . "./code.cg2js/cg-plot-widget.fasl")
(#x4810b700378b90826dd4b4d460a5a368 . "./code.cg2js/cg-pixmap.fasl")
(#x9b931117be18e06a372b55c00c983126 . "./code.cg2js/cg-font-dialog.fasl")
(#xe51e6a3b850974ceb557e4b5d5e6bb7a . "./code.cg2js/cg-os-widget.fasl")
(#x7b16758b65ee057753751f417a5ecf2f . "./code.cg2js/cg-text-or-combo.fasl")
(#x9dd06d85cb6d59373813e02eb7a2c183 . "./code.cg2js/cg-trackbar.fasl")
(#xf4b7b5784d39dd467e43185fe17ce384 . "./code.cg2js/cg-up-down-control.fasl")
(#x1cf0ef3e56433d8d4db9424501d49542 . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#xb0ac0bda79fd9078f893f155f44c734c . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#xa793c160652ec371569cf3e4e60ff6ce . "./code.cg2js/cg-intersect-posbox.fasl")
(#x81dd893c515b17a00dc6b5d4ab0b4e9e . "./code.cg2js/cg-bitmap-stream.fasl")
(#x82d814e084cc0a81a6e833a16036d079 . "./code.cg2js/cg-chart-widget.fasl")
(#x4705d44d01c82851d92b788cd5e3c8f5 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#x1e566abfb06c498eb0b4ff6a96418a9a . "./code.cg2js/cg-group-box.fasl")
(#x61ab9eee3e3f4734b3c0d4d8ef48f204 . "./code.cg2js/cg-static-text.fasl")
(#x68549779a9b0c4592ccfc9eea56acbac . "./code.cg2js/cg-class-slot-grid.fasl")
(#xb18aba3828c6b20da9eb22fad47c9dab . "./code.cg2js/cg-color-dialog.fasl")
(#xfbc1a78e601e37221c1d296285edf381 . "./code.cg2js/cg-intersect.fasl")
(#x0049888eac626bdf153f77c516b8bb6b . "./code.cg2js/cg-gesture-emulation.fasl")
(#x9f5c13587cce544eaed304496d88a321 . "./code.cg2js/cg-drawable-clipboard.fasl")
(#xbc841c234559508ab5312fa35efcea7b . "./code.cg2js/cg-html-dialog.fasl")
(#x3dbc74f9f69d63889164fb8d4c60bda7 . "./code.cg2js/cg-lettered-menu.fasl")
(#x7673f4e02ed90b3406895049ac9132e4 . "./code.cg2js/cg-class-grid.fasl")
(#x61ee580cea5d4447c282bf916a14ee61 . "./code.cg2js/cg-pixmap-printing.fasl")
(#xa38dd3fe37eab7e99bae4af02fdbde1a . "./code.cg2js/cg-calendar.fasl")
(#x3bd022c843529d4760417ad1665980b6 . "./code.cg2js/cg-editable-text.fasl")
(#x131c40fb90f6d3c58ba6df738c0233b3 . "./code.cg2js/cg-property.fasl")
(#x5a7e98da918332f49d460b157b3acb0b . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#x9e2b91ea3f258fb66e79bdc85e00b9e6 . "./code.cg2js/cg-grid-widget.fasl")
(#xd28078686ee19fdc65b8101158ef0bb3 . "./code.cg2js/cg-paren-matching.fasl")
(#xe7f4ef95e3794d8741d74f2a7538342d . "./code.cg2js/cg-dev.fasl")
(#x008a5c3b705957f8da830ec0e18909c3 . "./code.cg2js/cg-drawable.fasl")
(#x860e39be3eb486799a789c72d7e458c2 . "./code.cg2js/cg-curve.fasl")
(#x98d91956600eeaf4b33f0df2a5526497 . "./code.cg2js/cg-lisp-widget.fasl")
(#x3f5bc4134f0eac116c98b2d4750c389e . "./code.cg2js/cg-dde-utils.fasl")
(#xe0fa425c85aefaff19ff309fa03eb375 . "./code.cg2js/cg-pixmap-file-io.fasl")
(#xbc4a9a33977bfd0fd9ae142b1d01e1a0 . "./code.cg2js/cg-object-editor-layout.fasl")
(#xe1ab5a9d789f209a319aeb0459d26a04 . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x93161daaa38b1462af84ccbc8a019e24 . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x41cab16912af1252f4715a2bb1679a99 . "./code.cg2js/cg-lamp.fasl")
(#x59a68f636e535db68c55ba9ec90a5921 . "./code.cg2js/cg-common-control.fasl")
(#xd8cb3d9df96d51f8d4eed3b051754539 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#xf2691b01a371637dbe7c2f823e0d0f13 . "./code.cg2js/cg-directory-dialog-os.fasl")
(#x5bc3535a42d8237a301d87c475e1afd3 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x13ab2ff0fb77ab04e6b928de90778c0e . "./code.cg2js/cg-project-window.fasl")
(#x6c30985c9d14c374ce7c58cfa1a5c864 . "./code.cg2js/cg-sample-file-menu.fasl")
(#x461894df1aeba0b2aeb5756b1e8ffb32 . "./code.cg2js/cg-class-support.fasl")
(#x2f81854e081e43c404e4f9af8fdf0b48 . "./code.cg2js/cg-multi-picture-button.fasl")
(#x979ef4279ad2ace6e55bbd5cf52cb4de . "./code.cg2js/cg-yes-no-list.fasl")
(#xdac2b478c3f869ca260817310b1aa681 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#x1411ce954041fc8f90190838f9a91739 . "./code.cg2js/cg-string-dialog.fasl")
(#xbb932feefe97ea6ddbee16e3f80a264c . "./code.cg2js/cg-menu-tooltip.fasl")
(#x203a40b9d7ae9034c0b1dc728b4a6ede . "./code.cg2js/cg-dropping-outline.fasl")
(#x15be09bd0426f9c59f4ba609ac0aaf48 . "./code.cg2js/cg.fasl")
(#xb31cc2a9f1b460735e066f96785b483a . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#xaea6d11c6c0f0c10d29a6eb76ae8ab09 . "./code.cg2js/cg-scaling-stream.fasl")
(#x7fb46edc9fcab2d278610ba523d6a7aa . "./code.cg2js/cg-yes-no-string.fasl")
(#x8d6a1d705e0646dd774ac1348608e05e . "./code.cg2js/cg-icon-pixmap.fasl")
(#xda918e0317bfc5b43c1d0841e76a43f5 . "./code.cg2js/cg-web-browser.fasl")
(#x48aff6d7e2ed750cae8bde9aa4d9e761 . "./code.cg2js/cg-clipboard-stack.fasl")
(#xbabc7180cc040be53bd6d87958472741 . "./code.cg2js/cg-drag-and-drop.fasl")
(#x0a0fd163aaf5471873b3186ef461438b . "./code.cg2js/cg-message-dialog.fasl")
(#x2bec18190704938523bf9fd026079b9b . "./code.cg2js/cg-comtab.fasl")
(#x307828223dbf5a602c55d7c59de7f87f . "./code.cg2js/cg-base.fasl")
(#x3a056d09b9c33599009cdd5c5d3152eb . "./code.cg2js/cg-pixmap-widget.fasl")
(#xce677eaa036b3d64f3df5772c216c3d8 . "./code.cg2js/cg-text-edit-pane.fasl")
(#xa3d9f9465864bd7ccc51c600b626bbfb . "./code.cg2js/cg-utility-dialog.fasl")
(#x24b0eddfc0a050f6aebd8f0c995903a6 . "./code.cg2js/cg-object-editor.fasl")
(#xbcb791bdc2005922d116043d043305ea . "./code.cg2js/cg-edit-in-place.fasl")
(#x29ce37d0b044d73f299e8c16b6bec10b . "./code.cg2js/cg-radio-button.fasl")
(#xb8a3b5d2342ae73c4791dc07dfd890b4 . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#x455121c0f1b505244cf2d76503c513a5 . "./code.cg2js/cg-nodes.fasl")
(#x6fa8cb9db6838c8e060c84e2d738828b . "./code.cg2js/cg-get-position.fasl")
(#xdf4305b13372d17b6c6f466ae44fa9c3 . "./code.cg2js/cg-directory-dialog.fasl")
(#x10051d425f54c10b460e8d0d7c569557 . "./code.cg2js/cg-lisp-text.fasl")
(#xaa5f523a53152bd0badba36757511b4e . "./code.cg2js/cg-combo-box.fasl")
(#xf019f3873ad9acae75ab4dd70016c13a . "./code.cg2js/cg-text-widget.fasl")
(#x1e6027a7afc63e3ab14e102c8b825f02 . "./code.cg2js/cg-list-view.fasl")
(#x2146e1471e92649dd5448a736d6fae69 . "./code.cg2js/cg-graphics-context.fasl")
(#xe8f9cf71e29d5919170db5e1d2fc733e . "./code.cg2js/cg-get-pixmap.fasl")
(#xe9168130480c4a885786e0383499edeb . "./code.cg2js/cg-menu.fasl")
(#x21e9e6fe3060ec7547743a6b3aa3f8f9 . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#xe0f8c3781a0bd1fe9557a8a49dc805cd . "./code.cg2js/cg-pixmap-rotate.fasl")
(#xdc49afd41b93a26c3cf633c4bf91aa7c . "./code.cg2js/cg-tooltip.fasl")
(#x4fd9d7df2e3ca9801086517bdd86c6da . "./code.cg2js/cg-html-widget.fasl")
(#x2afa6c336f0735b5c2d92f0a628b1193 . "./code.cg2js/cg-choice-list.fasl")
(#xc2b6a49a4974aa5ae7d0efc39a8c3552 . "./code.cg2js/cg-icon.fasl")
(#x599acf8f2dee899d0c289fc56e90dd10 . "./code.cg2js/cg-acache.fasl")
(#xb851824a956011b55970673209286301 . "./code.cg2js/cg-template-string.fasl")
(#x8a4ce8f115f3bc660cb0479591c55678 . "./code.cg2js/cg-bitmap-pane.fasl")
(#x46760412a1ab3061281c33378650ec46 . "./code.cg2js/cg-toggling-widget.fasl")
(#xe42a12b1ecc50c10833bda5771752136 . "./code.cg2js/cg-tab-control.fasl")
(#xc8ca096c1af62bd52b1714e01b5eaf6e . "./code.cg2js/cg-split-bar.fasl")
(#x1e835db875cb75139c87148a9a258a24 . "./code.cg2js/cg-picture-widget.fasl")
(#xe5b949f6b59de35798f42245042c45b8 . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x47a682ca8aa2faf57ecf8ac4ef382b65 . "./code.cg2js/cg-choose-printer.fasl")
(#xc6ed4779c7e9182e866edf13b3056bce . "./code.cg2js/cg-os-window.fasl")
(#xc458b79d28f26a95c8f9818184e0b1f1 . "./code.cg2js/cg-scroll-bar.fasl")
(#x340d93e2bbfff2535daf2d55884bb95f . "./code.cg2js/cg-clipboard.fasl")
(#x48b2a6b5e4dae27c6d94464914abe7ad . "./code.cg2js/cg-find-string-dialog.fasl")
(#xf1ffa9de7722f2aaef131f5c749f558d . "./code.cg2js/cg-hotspot.fasl")
(#xb27a2f5788a27fbb01030957029e401a . "./code.cg2js/cg-check-box.fasl")
(#x4c7249b3fff2bed6c1c0c0452dae457e . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#x0f4453e678804a768f768d758eefd52f . "./code.cg2js/cg-file-dialog.fasl")
(#x276ebf0eee143661913fe8119adac21b . "./code.cg2js/cg-toolbar.fasl")
(#x83b535c7464b4ab1522857ee62243942 . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#x4d938f88dfad68448f042ad9fdf8aa33 . "./code.cg2js/cg-chart-or-plot.fasl")
(#x067555305412b3068f5156e207dcc5ea . "./code.cg2js/cg-shortcut-menu.fasl")
(#x58ad59734c5c927be8ee339352f36f5f . "./code.idejs/ide-project.fasl")
(#x0963925f6053b4f0ac3e6f5f9aff8f9a . "./code.idejs/ide-intro.fasl")
(#x2b327f0e099af0274ea463e189dd474c . "./code.idejs/ide-find-in-files.fasl")
(#xbdefcba0bfb7420680e0c5973674ba70 . "./code.idejs/ide-grapher.fasl")
(#x1b11a381ea0f771f7ee711a260dbaba1 . "./code.idejs/ide-inspector.fasl")
(#xa99b80073de51e1f6cd86efb0e2dbfc6 . "./code.idejs/ide-editor.fasl")
(#xd6da4206d923939c70e9a73aa34fb938 . "./code.idejs/ide-window-list.fasl")
(#xd893b3d911cb305befd1b9c0cc5dde7b . "./code.idejs/ide-process-browser.fasl")
(#x5ef6fdb7d5af48517c6ff93a2f84ec3c . "./code.idejs/ide-patch-dialog.fasl")
(#x7d017917cdcb8b88543b6aa0576856c7 . "./code.idejs/ide-class-grapher.fasl")
(#xe5a0185680293aad1fd9e9d2678bd50a . "./code.idejs/ide-base.fasl")
(#x29a3209b88bbcb8250647de0394fe84a . "./code.idejs/ide.fasl")
(#xf252e12a11cc4f1670b3e7c7dc608ac9 . "./code.idejs/ide-trace.fasl")
(#x913ff0be600e0324cd6a7eec8a7134ed . "./code.idejs/ide-form.fasl")
(#x116c5bfb9642289eca420bbd6f360780 . "./code.idejs/ide-find-definitions.fasl")
(#x76f216f13cbc9e3a831750a960e87afb . "./code.idejs/ide-apropos.fasl")
(#x2a2c390b0db58bc4884b9cdaa0bccaed . "./code.idejs/ide-class-browser.fasl")
(#xab8658f51881a5d16b1088aa660ceacc . "./code.idejs/ide-navigator.fasl")
(#x01e7fd1f09e9304628be911bd6bd7485 . "./code.idejs/ide-clipboard.fasl")
(#x3a5b0accea08fccdeab6912af508acb1 . "./code.idejs/ide-acache.fasl")
(#x21afcab9d60c26342e7c289e858d5ce8 . "./code.idejs/ide-keyshort.fasl")
(#xccfe7045323458083ea7a1296e899e4f . "./code.idejs/ide-package-list.fasl")
(#x537e9d824fd1af6e22300552e31e38a6 . "./code.idejs/ide-dev.fasl")
(#x49a17b9a618997e974ff2e9af0f2eb06 . "./cgjs.html")
(#x44f169492307df5920f609d11cc1a34c . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
